/* Gnome-Streamer
 * Copyright (C) <2002> Rehan Khwaja <rehankhwaja@yahoo.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* First, include the header file for the plugin, to bring in the
 * object definition and other useful things.
 */
#include <string.h>
#include <fcntl.h>
#include "gstdxr3video.h"
#include <sys/ioctl.h>
#include <linux/em8300.h>

/* Arguments are identified the same way, but cannot be zero, so you
 * must leave the ARG_0 entry in as a placeholder.
 */
enum {
  ARG_0,
  ARG_TV_MODE,
  ARG_DEVICE,
  ARG_ASPECT_RATIO
  /* TODO -
   * DIGITAL/ANALOG AUDIO OUT
   * OVERLAY MODE
   * SET BCS??  what's that?
   * OVERLAY ATTRIBUTES - where on the screen to put image
   * OVERLAY WINDOW - as above
   * OVERLAY KEYCOLOR
   * OVERLAY SIGNAL MODE
   * PLAYMODE - pausing, playing, slow forward etc
   */
};

/* A number of functon prototypes are given so we can refer to them later. */
static void	gst_dxr3video_class_init	(GstDxr3videoClass *klass);
static void	gst_dxr3video_init		(GstDxr3video *example);

static void	gst_dxr3video_chain		(GstPad *pad, GstBuffer *buf);

static void	gst_dxr3video_set_property	(GObject *object, guint prop_id,
						 const GValue *value, GParamSpec *pspec);
static void	gst_dxr3video_get_property	(GObject *object, guint prop_id,
						 GValue *value, GParamSpec *pspec);
static gboolean	gst_dxr3video_handle_event	(GstPad *pad, GstEvent *event);
static void	gst_dxr3video_set_clock		(GstElement *element, GstClock *clock);

/* The parent class pointer needs to be kept around for some object
 * operations.
 */
static GstElementClass *parent_class = NULL;

#define GST_TYPE_DXR3VIDEO_ASPECT_RATIOS (gst_dxr3video_aspect_ratios_get_type())
static GType
gst_dxr3video_aspect_ratios_get_type(void) {
  static GType dxr3video_aspect_ratio_type = 0;
  static GEnumValue dxr3video_aspect_ratios[] = {
    {0, "0", "4:3"},
    {1, "1", "16:9"},
    {0, NULL, NULL},
  };
  if (!dxr3video_aspect_ratio_type) {
    dxr3video_aspect_ratio_type = g_enum_register_static("GstDxr3VideoAspectRatios", dxr3video_aspect_ratios);
  }
  return dxr3video_aspect_ratio_type;
}

#define GST_TYPE_DXR3VIDEO_TV_MODES (gst_dxr3video_tv_modes_get_type())
static GType
gst_dxr3video_tv_modes_get_type(void) {
  static GType dxr3video_tv_mode_type = 0;
  static GEnumValue dxr3video_tv_modes[] = {
    {0, "0", "NTSC"},
    {1, "1", "PAL"},
    {0, NULL, NULL},
  };
  if (!dxr3video_tv_mode_type) {
    dxr3video_tv_mode_type = g_enum_register_static("GstDxr3VideoTvModes", dxr3video_tv_modes);
  }
  return dxr3video_tv_mode_type;
}

static void
gst_dxr3video_set_clock(GstElement *element, GstClock *clock)
{
  GstDxr3video *dxr3video;

  dxr3video = GST_DXR3VIDEO(element);
  dxr3video->clock = clock;
}

/* This function is used to register and subsequently return the type
 * identifier for this object class.  On first invocation, it will
 * register the type, providing the name of the class, struct sizes,
 * and pointers to the various functions that define the class.
 */
GType
gst_dxr3video_get_type(void)
{
  static GType dxr3video_type = 0;

  if (!dxr3video_type) {
    static const GTypeInfo dxr3video_info = {
      sizeof(GstDxr3videoClass),      NULL,
      NULL,
      (GClassInitFunc)gst_dxr3video_class_init,
      NULL,
      NULL,
      sizeof(GstDxr3video),
      0,
      (GInstanceInitFunc)gst_dxr3video_init,
    };
    dxr3video_type = g_type_register_static(GST_TYPE_ELEMENT, "GstDxr3video", &dxr3video_info, 0);
  }
  return dxr3video_type;
}

static gchar *
gst_dxr3video_get_control_device(GstDxr3video *dxr3video)
{
  return g_strdup_printf("/dev/em8300-%d", dxr3video->device_number);
}

static gchar *
gst_dxr3video_get_video_device(GstDxr3video *dxr3video)
{
  return g_strdup_printf("/dev/em8300_mv-%d", dxr3video->device_number);
}

/* In order to create an instance of an object, the class must be
 * initialized by this function.  GObject will take care of running
 * it, based on the pointer to the function provided above.
 */
static void
gst_dxr3video_class_init (GstDxr3videoClass *klass)
{
  /* Class pointers are needed to supply pointers to the private
   * implementations of parent class methods.
   */
  GObjectClass *gobject_class;
  GstElementClass *gstelement_class;

  /* Since the example class contains the parent classes, you can simply
   * cast the pointer to get access to the parent classes.
   */
  gobject_class = (GObjectClass*)klass;
  gstelement_class = (GstElementClass*)klass;

  /* The parent class is needed for class method overrides. */
  parent_class = g_type_class_ref(GST_TYPE_ELEMENT);

  /* Here we add an argument to the object.  This argument is an integer,
   * and can be both read and written.
   */
  g_object_class_install_property(G_OBJECT_CLASS(klass), ARG_TV_MODE,
    g_param_spec_enum("tv-mode","tv-mode","sets NTSC/PAL output format",
                     GST_TYPE_DXR3VIDEO_TV_MODES, 0, G_PARAM_READWRITE));

  g_object_class_install_property(G_OBJECT_CLASS(klass), ARG_DEVICE,
    g_param_spec_int("device","device","sets/returns board number",
                     0,3,0,G_PARAM_READWRITE));

  g_object_class_install_property(G_OBJECT_CLASS(klass), ARG_ASPECT_RATIO,
    g_param_spec_enum("aspect-ratio", "aspect-ratio", "sets/returns aspect ratio",
                     GST_TYPE_DXR3VIDEO_ASPECT_RATIOS, 0, G_PARAM_READWRITE));

  /* The last thing is to provide the functions that implement get and set
   * of arguments.
   */
  gobject_class->set_property = gst_dxr3video_set_property;
  gobject_class->get_property = gst_dxr3video_get_property;
}

static void
gst_dxr3video_open_device(GstDxr3video *dxr3video)
{
  gchar *video_device_path;

  video_device_path = gst_dxr3video_get_video_device(dxr3video);
  if (dxr3video->device){
    fclose(dxr3video->device);
  }
  /* this is what disksink uses */
  dxr3video->device = fopen(video_device_path, "w");

  g_free(video_device_path);
}

/* This function is responsible for initializing a specific instance of
 * the plugin.
 */
static void
gst_dxr3video_init(GstDxr3video *dxr3video)
{
  /* First we create the sink pad, which is the input to the element.
   * We will use the template constructed by the factory.
   */
  dxr3video->sinkpad = gst_pad_new_from_template (
		  GST_PADTEMPLATE_GET (dxr3video_factory), "video_sink");
  /* Setting the chain function allows us to supply the function that will
   * actually be performing the work.  Without this, the element would do
   * nothing, with undefined results (assertion failures and such).
   */
  gst_pad_set_chain_function(dxr3video->sinkpad, gst_dxr3video_chain);
  /* We then must add this pad to the element's list of pads.  The base
   * element class manages the list of pads, and provides accessors to it.
   */
  gst_element_add_pad(GST_ELEMENT(dxr3video),dxr3video->sinkpad);

  /* Initialization of element's private variables. */
  dxr3video->device_number = 0;
  dxr3video->device = 0;
  /* FIXME - should only have device open when necessary
   * which probably requires handling events?
   * also, when is this device closed?
   */
  gst_dxr3video_open_device(dxr3video);

  GST_FLAG_SET (GST_ELEMENT(dxr3video), GST_ELEMENT_EVENT_AWARE);
  gst_pad_set_event_function(dxr3video->sinkpad, gst_dxr3video_handle_event);

  dxr3video->clock = NULL;
  GST_ELEMENT(dxr3video)->setclockfunc = gst_dxr3video_set_clock;
}

static gboolean
gst_dxr3video_handle_event(GstPad *pad, GstEvent *event)
{
  GstEventType type;

  type = event ? GST_EVENT_TYPE (event) : GST_EVENT_UNKNOWN;

  switch (type) {
    case GST_EVENT_SEEK:
      g_print("seek event\n\n");
      break;
    case GST_EVENT_NEW_MEDIA:
      g_print("new media event\n\n");
      break;
    case GST_EVENT_FLUSH:
      g_print("flush event\n\n");
      break;
    default:
      g_print("event\n\n");
      gst_pad_event_default (pad, event);
      break;
  }

  return TRUE;
}

/* The chain function is the heart of the element.  It's where all the
 * work is done.  It is passed a pointer to the pad in question, as well
 * as the buffer provided by the peer element.
 */
static void
gst_dxr3video_chain (GstPad *pad, GstBuffer *buf)
{
  GstDxr3video *dxr3video;
  long pts;

  /* Some of these checks are of dubious value, since if there were not
   * already true, the chain function would never be called.
   */
  g_return_if_fail(pad != NULL);
  g_return_if_fail(GST_IS_PAD(pad));
  g_return_if_fail(buf != NULL);

  /* We need to get a pointer to the element this pad belogs to. */
  dxr3video = GST_DXR3VIDEO(gst_pad_get_parent (pad));

  /* A few more sanity checks to make sure that the element that owns
   * this pad is the right kind of element, in case something got confused.
   */
  g_return_if_fail(dxr3video != NULL);
  g_return_if_fail(GST_IS_DXR3VIDEO(dxr3video));

  /* Then copy the data in the incoming buffer onto the device. */
  fwrite(GST_BUFFER_DATA (buf), 1, GST_BUFFER_SIZE (buf), dxr3video->device);
  pts = (long)GST_BUFFER_TIMESTAMP(buf);
  if (-1 == ioctl((int)dxr3video->device, EM8300_IOCTL_VIDEO_SETPTS, &pts)){
    GST_DEBUG(0, "FAILED call to EM8300_IOCTL_VIDEO_SETPTS\n");
  }

  gst_buffer_unref (buf);
}

/* Arguments are part of the Gtk+ object system, and these functions
 * enable the element to respond to various arguments.
 */
static void
gst_dxr3video_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
{
  GstDxr3video *dxr3video;
  int device;
  gchar *device_path;
  int tv_mode_ioctl, aspect_ratio_ioctl;

  /* It's not null if we got it, but it might not be ours */
  g_return_if_fail(GST_IS_DXR3VIDEO(object));

  /* Get a pointer of the right type. */
  dxr3video = GST_DXR3VIDEO(object);

  /* Check the argument id to see which argument we're setting. */
  switch (prop_id) {
    case ARG_TV_MODE:
      device_path = gst_dxr3video_get_control_device(dxr3video);
      device = open(device_path, O_WRONLY);
      if (device == -1){
        GST_DEBUG(0, "failed to open control device\n");
	break;
      }
      switch(g_value_get_int(value)){
      case 0:
        tv_mode_ioctl = EM8300_VIDEOMODE_NTSC;
	break;
      case 1:
        tv_mode_ioctl = EM8300_VIDEOMODE_PAL;
	break;
      case 2:
        tv_mode_ioctl = EM8300_VIDEOMODE_PAL60;
	break;
      }
      if (-1 == ioctl(device, EM8300_IOCTL_SET_VIDEOMODE, &tv_mode_ioctl)){
        GST_DEBUG (0,"failed to set tv-mode\n");
      }
      close(device);
      break;
    case ARG_DEVICE:
      dxr3video->device_number = g_value_get_int(value);
      gst_dxr3video_open_device(dxr3video);
      break;
    case ARG_ASPECT_RATIO:
      device_path = gst_dxr3video_get_control_device(dxr3video);
      device = open(device_path, O_WRONLY);
      if (device == -1){
        GST_DEBUG(0, "failed to open control device\n");
	break;
      }
      switch(g_value_get_enum(value)){
      case 0:
        aspect_ratio_ioctl = EM8300_ASPECTRATIO_4_3;
	break;
      case 1:
        aspect_ratio_ioctl = EM8300_ASPECTRATIO_16_9;
	break;
      }
      if (-1 == ioctl(device, EM8300_IOCTL_SET_ASPECTRATIO, &aspect_ratio_ioctl)){
        GST_DEBUG(0, "failed to set aspect-ratio\n");
      }
      break;
    default:
      break;
  }
}

/* The set function is simply the inverse of the get fuction. */
static void
gst_dxr3video_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec)
{
  GstDxr3video *dxr3video;
  gint device;
  gchar *device_path;
  int tv_mode_ioctl, aspect_ratio_ioctl;

  /* It's not null if we got it, but it might not be ours */
  g_return_if_fail(GST_IS_DXR3VIDEO(object));
  dxr3video = GST_DXR3VIDEO(object);

  switch (prop_id) {
    case ARG_TV_MODE:
      device_path = gst_dxr3video_get_control_device(dxr3video);
      tv_mode_ioctl = 0;
      device = open(device_path, O_WRONLY);
      if (-1 == device){
        GST_DEBUG(0, "failed to open control device\n");
	break;
      }
      if (-1 == ioctl(device, EM8300_IOCTL_GET_VIDEOMODE, &tv_mode_ioctl)){
        GST_DEBUG(0, "failed to get tv-mode\n");
      }
      close(device);

      switch(tv_mode_ioctl){
      case EM8300_VIDEOMODE_NTSC:
        g_value_set_enum(value, 0);
	break;
      case EM8300_VIDEOMODE_PAL:
        g_value_set_enum(value, 1);
	break;
      case EM8300_VIDEOMODE_PAL60:
        g_value_set_enum(value, 2);
	break;
      }
      break;
    case ARG_DEVICE:
      g_value_set_int(value, dxr3video->device_number);
      break;
    case ARG_ASPECT_RATIO:
      device_path = gst_dxr3video_get_control_device(dxr3video);
      aspect_ratio_ioctl = 0;
      device = open(device_path, O_WRONLY);
      if (-1 == device){
        GST_DEBUG(0, "failed to open control device\n");
	break;
      }
      if (-1 == ioctl(device, EM8300_IOCTL_GET_ASPECTRATIO, &aspect_ratio_ioctl)){
        GST_DEBUG(0, "failed to get aspect ratio\n");
      }
      close(device);

      switch(aspect_ratio_ioctl){
      case EM8300_ASPECTRATIO_4_3:
        g_value_set_enum(value, 0);
	break;
      case EM8300_ASPECTRATIO_16_9:
        g_value_set_enum(value, 1);
        break;
      }
      break;
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
      break;
  }
}
