/* Gnome-Streamer
 * Copyright (C) <2002> Rehan Khwaja <rehankhwaja@yahoo.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#ifndef __GSTDXR3VIDEO_H__
#define __GSTDXR3VIDEO_H__

#include <gst/gst.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


/* This is the definition of the element's object structure. */
typedef struct _GstDxr3video GstDxr3video;
/* forward declaration */
struct _GstDxr3video;

/* The structure itself is derived from GstElement, as can be seen by the
 * fact that there's a complete instance of the GstElement structure at
 * the beginning of the object.  This allows the element to be cast to
 * an Element or even an Object.
 */
struct _GstDxr3video {
  GstElement element;

  //board number
  gint device_number;

  /* file handle for the video device */
  FILE *device;

  GstClock *clock;

  /* our only pad */
  GstPad *sinkpad;
};

/* The other half of the object is its class.  The class also derives from
 * the same parent, though it must be the class structure this time.
 * Function pointers for polymophic methods and signals are placed in this
 * structure. */
typedef struct _GstDxr3videoClass GstDxr3videoClass;

struct _GstDxr3videoClass {
  GstElementClass parent_class;
};

/* Five standard preprocessing macros are used in the Gtk+ object system.
 * The first uses the object's _get_type function to return the GType
 * of the object.
 */
#define GST_TYPE_DXR3VIDEO \
  (gst_dxr3video_get_type())
/* The second is a checking cast to the correct type.  If the object passed
 * is not the right type, a warning will be generated on stderr.
 */
#define GST_DXR3VIDEO(obj) \
  (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_DXR3VIDEO,GstDxr3video))
/* The third is a checking cast of the class instead of the object. */
#define GST_DXR3VIDEO_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_DXR3VIDEO,GstDxr3video))
/* The last two simply check to see if the passed pointer is an object or
 * class of the correct type. */
#define GST_IS_DXR3VIDEO(obj) \
  (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_DXR3VIDEO))
#define GST_IS_DXR3VIDEO_CLASS(obj) \
  (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_DXR3VIDEO))

/* This is the only prototype needed, because it is used in the above
 * GST_TYPE_DXR3VIDEO macro.
 */
GType gst_dxr3video_get_type(void);

/* The PadFactory structures describe what pads the element has or
 * can have.  They can be quite complex, but for this example plugin
 * they are rather simple.
 */
GST_PADTEMPLATE_FACTORY (dxr3video_factory,
  "sink",			/* The name of the pad */
  GST_PAD_SINK,		/* Direction of the pad */
  GST_PAD_ALWAYS,	/* The pad exists for every instance */
  gst_caps_new("video_sink", "video/mpeg", NULL)
);

#ifdef __cplusplus
}
#endif /* __cplusplus */


#endif /* __GSTDXR3VIDEO_H__ */
